/**
 *
 */
package de.renew.ant;

import org.apache.log4j.Logger;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

import CH.ifa.draw.framework.Drawing;

import CH.ifa.draw.io.DrawingFileHelper;
import CH.ifa.draw.io.StatusDisplayer;

import de.renew.io.exportFormats.EPSExportFormat;

import java.io.File;

import java.util.Iterator;
import java.util.Vector;


/**
 * Ant task to create EPS graphics from Renew drawings
 *
 * @author 9markwar
 *
 */
public class CreateNetEpsTask extends Task {
    private File destdir = null;
    private Vector filesets = new Vector();

    /**
     *
     */
    public CreateNetEpsTask() {
        super();
    }

    /**
     * Configures the root directory of the directory tree where all
     * <code>.eps</code> files are put into.
     *
     * @param dest a directory location
     **/
    public void setDestdir(File dest) {
        this.destdir = dest;
    }

    /**
     * Is invoked by the ant task and exports all the given
     * <code>.rnw<code/> to <code>.eps<code/>
     * into the same directory as the original file is in.
     **/
    public void execute() throws BuildException {
        super.execute();
        EPSExportFormat epsExportFormat = new EPSExportFormat(); // object helps to export .rnw into .eps

        // get all files - out of the vector full of filesets -> 2 loops
        Iterator filesetIterator = filesets.iterator();
        while (filesetIterator.hasNext()) {
            FileSet elementFileSet = (FileSet) filesetIterator.next();
            DirectoryScanner dirScan = elementFileSet.getDirectoryScanner(getProject());
            String[] fileNames = dirScan.getIncludedFiles();
            log("filenames: " + fileNames.length, Project.MSG_VERBOSE);

            for (int i = 0; i < fileNames.length; i++) {
                String fileName = fileNames[i];
                File inFile = new File(dirScan.getBasedir() + "/" + fileName);
                log("File: " + inFile.getAbsolutePath(), Project.MSG_VERBOSE);

                // get drawing from file
                Drawing netDrawing = DrawingFileHelper.loadDrawing(inFile,
                                                                   new StatusDisplayer() {
                        public void showStatus(String message) {
                            log(message);
                        }
                    }); //rnw-Object
                log("Drawing: " + netDrawing, Project.MSG_VERBOSE);

                // Export netDrawing to EPS
                try {
                    File helpFile = new File(destdir + "/" + fileName); // helps to get the right directory path
                    File outFile = new File(helpFile.getParent() + "/"
                                            + netDrawing.getName() + ".eps");

                    //create directories
                    outFile.getParentFile().mkdirs();
                    epsExportFormat.export(netDrawing, outFile);
                } catch (Exception e) {
                    throw new BuildException(e);
                }
            }
        }
    }

    public void addFileset(FileSet fileset) {
        filesets.add(fileset);
    }
}