package de.renew.plugin.command;

import de.renew.plugin.IPlugin;
import de.renew.plugin.PluginManager;

import java.io.PrintStream;

import java.net.MalformedURLException;
import java.net.URL;


/**
 * This command loads a plugin from the given URL.
 * If the argument "-v" is given, verbose output is printed.
 *
 * @author J&ouml;rn Schumacher
 *
 */
public class LoadCommand implements CLCommand {
    public static org.apache.log4j.Logger logger = org.apache.log4j.Logger
                     .getLogger(LoadCommand.class);

    public void execute(String[] args, PrintStream response) {
        boolean verbose = false;
        URL url = null;
        if (args.length != 1 || "h".equals(args[0])) {
            response.println("usage: load <url>\n" + "examples: \n"
                             + "load file:/path/to/location/name/plugin.cfg\n"
                             + "load file:/path/to/location/pluginname.jar");
        } else {
            response.println("looking for plugin at " + args[0] + " ...");
            try {
                url = new URL(args[0]);

                IPlugin plugin = PluginManager.getInstance().loadPlugin(url);
                if (plugin != null) {
                    response.println("loaded plugin: " + plugin.getName() + ".");
                } else {
                    response.println("Plugin not loaded. See log messages for details.");
                }
            } catch (MalformedURLException e) {
                response.println("Malformed URL: " + e.getMessage());
            } catch (Exception e) {
                response.println("load failed: " + e);
                logger.error(e.getMessage(), e);
            }
        }
    }

    public String getDescription() {
        return "loads a new plugin given a URL";
    }
}