package de.renew.plugin.locate;

import de.renew.plugin.PluginProperties;
import de.renew.plugin.PropertyHelper;

import java.net.URL;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;


/**
 * A Composition of PluginLocationFinders so several Finders can be used
 * to look for plugin configuration objects.
 */
public class PluginLocationFinders implements PluginLocationFinder {
    private static PluginLocationFinders _instance;
    private Collection _finders = new Vector();

    private PluginLocationFinders() {
    }

    public static PluginLocationFinders getInstance() {
        if (_instance == null) {
            _instance = new PluginLocationFinders();
        }
        return _instance;
    }

    public void addLocationFinder(PluginLocationFinder f) {
        _finders.add(f);
    }

    public void removeLocationFinder(PluginLocationFinder f) {
        _finders.remove(f);
    }

    /**
     * Calls findPluginLocations() for all contained PluginLocationFinders,
     * merging the result lists.
     */
    public Collection findPluginLocations() {
        Collection result = new Vector();
        Iterator i = _finders.iterator();
        PluginLocationFinder finder = null;
        while (i.hasNext()) {
            finder = (PluginLocationFinder) i.next();
            //    		logger.debug (finder + " looking for Pluginlocations");
            Collection locations = finder.findPluginLocations();
            result.addAll(locations);
        }

        // remove all items that the user didnt want to load
        Collection noLoad = PropertyHelper.parseListString(PluginProperties.getUserProperties()
                                                                           .getProperty("de.renew.plugin.noLoad",
                                                                                        ""));
        Iterator cleanedList = result.iterator();
        while (cleanedList.hasNext()) {
            PluginProperties props = (PluginProperties) cleanedList.next();
            if (noLoad.contains(props.getName())) {
                cleanedList.remove();
            }
        }

        return result;
    }

    /**
     * Calls <code>checkPluginLocation(url)</code> for all contained
     * <code>PluginLocationFinder</code>s, returning the first result
     * that differs from <code>null</code> (if there is any).
     **/
    public PluginProperties checkPluginLocation(URL url) {
        PluginProperties props = null;
        Iterator i = _finders.iterator();
        while ((props == null) && i.hasNext()) {
            PluginLocationFinder finder = (PluginLocationFinder) i.next();

            // logger.debug (finder + " checking plugin location");
            props = finder.checkPluginLocation(url);
        }
        return props;
    }
}