package de.renew.netcomponents;

import CH.ifa.draw.figures.TextFigure;

import CH.ifa.draw.framework.ConnectionFigure;
import CH.ifa.draw.framework.Drawing;
import CH.ifa.draw.framework.DrawingView;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;
import CH.ifa.draw.framework.UndoableCommand;

import CH.ifa.draw.standard.ReverseFigureEnumerator;

import de.renew.gui.GuiPlugin;
import de.renew.gui.PlaceFigure;
import de.renew.gui.TransitionFigure;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;


/**
 * Command to group the selection into a NetComponentFigure.
 *
 * @see GroupFigure
 */
public class GroupCommand extends UndoableCommand {
    // protected DrawingEditor fEditor;

    /**
     * Constructs a group command.
     */
    public GroupCommand() {
        super("group selection");
        // fEditor = editor;
    }

    public boolean executeUndoable() {
        DrawingView view = GuiPlugin.getCurrent().getDrawingEditor().view();
        Vector selected = view.selectionZOrdered();
        HashSet selection = new HashSet();
        Drawing drawing = view.drawing();
        if (selected.size() > 0) {
            view.clearSelection();
            Iterator iter = selected.iterator();
            while (iter.hasNext()) {
                Figure figure = (Figure) iter.next();
                if (!(figure instanceof NetComponentFigure)) {
                    selection.add(figure);
                }
            }

            // find all NetComponentFigures that are involved with the current selection. 
            HashSet ncFiguresInvolvedInSelection = getInvolvedNCFigures(drawing,
                                                                        selected);
            System.out.println("Size is " + ncFiguresInvolvedInSelection.size());
            // add all elements of the NCFs to the selection and remove the NCFs from the drawing
            Iterator it = ncFiguresInvolvedInSelection.iterator();
            while (it.hasNext()) {
                NetComponentFigure ncFigure = (NetComponentFigure) it.next();
                selection.addAll(ncFigure.getAttached());
                drawing.remove(ncFigure);
            }
            drawing.orphanAll(selected);

            drawing.orphanAll(new Vector(selection));


            // devide the selected figures into three devisions, so that they can be layered
            // Text and Connections on top then Transitions and Places then
            // the invisible NetComponentFigure and in the back all the rest.
            Vector first = new Vector();
            Vector second = new Vector();
            Vector third = new Vector();
            System.out.println("selection " + selection);
            it = selection.iterator();
            while (it.hasNext()) {
                Figure figure = (Figure) it.next();
                if (figure instanceof TextFigure
                            || figure instanceof ConnectionFigure) {
                    third.add(figure);
                } else if (figure instanceof TransitionFigure
                                   || figure instanceof PlaceFigure) {
                    second.add(figure);

                } else {
                    first.add(figure);
                }
            }
            NetComponentFigure group = new NetComponentFigure();
            group.group(selection);


            // now add all the figures to the drawing in the predefined order
            Vector v = new Vector();
            v.addAll(first);
            System.out.println("Netcomponentfigure " + group);
            System.out.println("Drawing " + drawing);
            v.add(group);
            v.addAll(second);
            v.addAll(third);
            drawing.addAll(v);


            //            selection.add(group);
            //            drawing.sendToBack(selection);
            FigureEnumeration k = new ReverseFigureEnumerator(v);
            while (k.hasMoreElements()) {
                view.drawing().sendToBack(k.nextFigure());
            }
            view.checkDamage();


            view.checkDamage();
            GuiPlugin.getCurrent()
                     .showStatus("Figures attachted fo NetComponentFigure");
            return true;
        }
        return false;
    }

    /**
     * @return
     */
    private HashSet getInvolvedNCFigures(Drawing drawing, Vector selected) {
        HashSet ncfs = new HashSet();
        FigureEnumeration it = drawing.figures();
        int i = 0;
        while (it.hasMoreElements()) {
            Figure figure = it.nextFigure();
            System.out.println("int i = " + i++);
            if (figure instanceof NetComponentFigure) {
                ncfs.add(figure);
                System.out.println("One NCFS is found " + i);

            }
        }
        HashSet involvedNCFigures = new HashSet();

        Iterator jt = ncfs.iterator();
        while (jt.hasNext()) {
            Figure figure = (Figure) jt.next();
            if (selected.contains(figure)) {
                involvedNCFigures.add(figure);
                System.out.println("NetComponentFigure is involved " + figure);
            }
            NetComponentFigure ncFigure = ((NetComponentFigure) figure);
            Iterator kt = ncFigure.getAttached().iterator();
            while (kt.hasNext()) {
                Figure attachedFigure = (Figure) kt.next();
                if (selected.contains(attachedFigure)) {
                    involvedNCFigures.add(ncFigure);
                }
            }
        }
        return involvedNCFigures;
    }

    public boolean isExecutable() {
        if (!super.isExecutable()) {
            return false;
        }
        return GuiPlugin.getCurrent().getDrawingEditor().view().selectionCount() > 0;
    }
}