/*
 * Created on Feb 12, 2004
 *
 * A NetComponentFigure is a virtual figure that hats no graphical
 * representation and no othe rfunction as to group a cople of other figures
 * into one loosely connected group. Mouse drags are being passed by such a
 * figure on to all its attached figures. Selections are being passed on to one
 * (or some) specific figures.
 */
package de.renew.netcomponents;

import CH.ifa.draw.DrawPlugin;

import CH.ifa.draw.application.DrawApplication;

import CH.ifa.draw.figures.AttributeFigure;
import CH.ifa.draw.figures.GroupHandle;
import CH.ifa.draw.figures.TextFigure;

import CH.ifa.draw.framework.ChildFigure;
import CH.ifa.draw.framework.DrawingView;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureChangeEvent;
import CH.ifa.draw.framework.FigureChangeListener;
import CH.ifa.draw.framework.FigureEnumeration;

import CH.ifa.draw.standard.FigureEnumerator;
import CH.ifa.draw.standard.MergedFigureEnumerator;
import CH.ifa.draw.standard.RelativeLocator;

import CH.ifa.draw.util.StorableInput;
import CH.ifa.draw.util.StorableOutput;

import java.awt.Point;
import java.awt.Rectangle;

import java.io.IOException;
import java.io.ObjectInputStream;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;


/**
 * @author Lawrence Cabac
 */
public class NetComponentFigure extends AttributeFigure
        implements FigureChangeListener { //RectangleFigure{//
    Vector attached = new Vector();


    /**
     * Group a vector of figures to form a NetComponentFigure.
     *
     * @param c -
     *            a collection of Figures that will be grouped.
     * @return the same collection
     */
    public Collection group(Collection c) {
        Iterator it = c.iterator();
        while (it.hasNext()) {
            Figure figure = (Figure) it.next();
            TextFigure f;
            if ((!(figure instanceof ChildFigure))
                        || ((ChildFigure) figure).parent() == null) {
                if (!attached.contains(figure)) {
                    attached.add(figure);
                    figure.addFigureChangeListener(this);
                }
            }
        }

        return c;
    }

    protected void basicMoveBy(int dx, int dy) {
        Vector selected = getView().selection();
        Iterator it = attached.iterator();
        while (it.hasNext()) {
            AttributeFigure af = (AttributeFigure) it.next();
            if (!selected.contains(af)) {
                af.moveBy(dx, dy);
            }
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.standard.AbstractFigure#basicDisplayBox(java.awt.Point,
     *      java.awt.Point)
     */
    public void basicDisplayBox(Point origin, Point corner) {
        // TODO Auto-generated method stub
    }

    /**
     * Gets the display box. The display box is defined as the union of the
     * contained figures.
     */
    public Rectangle displayBox() {
        Enumeration k = attached.elements();
        Rectangle r = new Rectangle();
        if (k.hasMoreElements()) {
            r = ((Figure) k.nextElement()).displayBox();
        }
        while (k.hasMoreElements()) {
            r.add(((Figure) k.nextElement()).displayBox());
        }
        if (r == null) {
            return new Rectangle(0, 0);
        }
        return r;
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.standard.AbstractFigure#handles()
     */


    /**
     * Gets the handles for the GroupFigure.
     */
    public Vector handles() {
        Vector handles = new Vector();
        handles.addElement(new GroupHandle(this, RelativeLocator.northWest()));
        handles.addElement(new GroupHandle(this, RelativeLocator.northEast()));
        handles.addElement(new GroupHandle(this, RelativeLocator.southWest()));
        handles.addElement(new GroupHandle(this, RelativeLocator.southEast()));
        return handles;
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.framework.Figure#moveBy(int, int)
     */
    public void moveBy(int dx, int dy) {
        //super.moveBy(dx, dy);
        Vector selected = getView().selection();
        Iterator it = attached.iterator();
        while (it.hasNext()) {
            Figure af = (Figure) it.next();
            if (!selected.contains(af)) {
                af.moveBy(dx, dy);
            }
        }
        changed();
    }

    /**
     * Returns whether this figure can be selected.
     */
    public boolean isSelectable() {
        return true;
    }

    private DrawingView getView() {
        DrawApplication gui = DrawPlugin.getGui();
        return gui.getView(gui.drawing());

    }

    /**
     * Stores the connector and its owner to a StorableOutput.
     */
    public void write(StorableOutput dw) {
        super.write(dw);
        dw.writeInt(attached.size());
        Iterator it = attached.iterator();
        while (it.hasNext()) {
            Figure figure = (Figure) it.next();
            dw.writeStorable(figure);

        }
    }

    /**
     * Reads the connector and its owner from a StorableInput.
     */
    public void read(StorableInput dr) throws IOException {
        if (dr.getVersion() > 9) {
            super.read(dr);
        }
        attached = new Vector();
        int size = dr.readInt();
        if (size != 0) {
            for (int i = 1; i <= size; i++) {
                Figure figure = (Figure) dr.readStorable();
                attached.add(figure);
                figure.addFigureChangeListener(this);
            }
        }
    }

    /**
     * @return a <code>Vector</code> of Figures that are attached to this
     *         <code>NetComponentFigure</code>.
     */
    public Vector getAttached() {
        return attached;
    }

    public boolean inspect(DrawingView view, boolean alternate) {
        if (alternate) {
            return super.inspect(view, alternate);
        } else {
            view.clearSelection();
            view.addToSelectionAll(attached);
            return true;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.framework.FigureChangeListener#figureInvalidated(CH.ifa.draw.framework.FigureChangeEvent)
     */
    public void figureInvalidated(FigureChangeEvent e) {
        // TODO Auto-generated method stub
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.framework.FigureChangeListener#figureChanged(CH.ifa.draw.framework.FigureChangeEvent)
     */
    public void figureChanged(FigureChangeEvent e) {
        // TODO Auto-generated method stub
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.framework.FigureChangeListener#figureRemoved(CH.ifa.draw.framework.FigureChangeEvent)
     */
    public void figureRemoved(FigureChangeEvent e) {
        Figure removed = e.getFigure();
        attached.remove(removed);
        if (attached.isEmpty() && listener() != null) {
            listener().figureRequestRemove(new FigureChangeEvent(this));
        }
        invalidate();
        changed();
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.framework.FigureChangeListener#figureRequestRemove(CH.ifa.draw.framework.FigureChangeEvent)
     */
    public void figureRequestRemove(FigureChangeEvent e) {
        // TODO Auto-generated method stub
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.framework.FigureChangeListener#figureRequestUpdate(CH.ifa.draw.framework.FigureChangeEvent)
     */
    public void figureRequestUpdate(FigureChangeEvent e) {
        // TODO Auto-generated method stub
    }

    /*
     * (non-Javadoc)
     *
     * @see CH.ifa.draw.framework.FigureChangeListener#figureHandlesChanged(CH.ifa.draw.framework.FigureChangeEvent)
     */
    public void figureHandlesChanged(FigureChangeEvent e) {
        // TODO Auto-generated method stub
    }

    //    /*
    //     * (non-Javadoc)
    //     * 
    //     * @see CH.ifa.draw.framework.Figure#release()
    //     */
    //    public void release() {
    //        Iterator it = attached.iterator();
    //        while (it.hasNext()) {
    //            Figure figure = (Figure) it.next();
    //            figure.removeFigureChangeListener(this);
    //        }
    //        super.release();
    //    }


    /**
     * Deserialization method, behaves like default readObject
     * method, but additionally restores the association from
     * contained figures to this composite figure.
     **/
    private void readObject(ObjectInputStream s)
            throws ClassNotFoundException, IOException {
        s.defaultReadObject();

        Enumeration k = attached.elements();
        while (k.hasMoreElements()) {
            Enumeration en = DrawPlugin.getGui().drawing().figures();
            Figure figure = (Figure) k.nextElement();
            while (en.hasMoreElements()) {
                Figure f = (Figure) en.nextElement();
                if (f.equals(figure)) {
                    figure.addToContainer(this);
                }
            }
        }
    }

    public FigureEnumeration getFiguresWithDependencies() {
        return new MergedFigureEnumerator(super.getFiguresWithDependencies(),
                                          getAttachedFigures());
    }


    /**
     * @return
     */
    private FigureEnumeration getAttachedFigures() {
        return new FigureEnumerator(attached);
    }


    /**
     *
     */
    public void updateListeners() {
        FigureEnumeration fenum = getFiguresWithDependencies();
        while (fenum.hasMoreElements()) {
            Figure figure = fenum.nextFigure();
            figure.addToContainer(this);
        }
    }
}