package de.renew.engine.common;

import de.renew.engine.searcher.AbstractOccurrence;
import de.renew.engine.searcher.Occurrence;
import de.renew.engine.searcher.OccurrenceDescription;
import de.renew.engine.searcher.Searcher;
import de.renew.engine.searcher.VariableMapperCopier;
import de.renew.engine.simulator.SimulationThreadPool;

import de.renew.net.TransitionInstance;

import de.renew.unify.Impossible;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;


/**
 * The base class for occurrences that derive their semantics from
 * a number of other occurrences.
 */
public class CompositeOccurrence extends AbstractOccurrence {
    private List occurrences = new ArrayList();

    public CompositeOccurrence(TransitionInstance tInstance) {
        super(tInstance);
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
    }

    public void addOccurrence(Occurrence occurrence) {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        occurrences.add(occurrence);
    }

    public void addOccurrences(Collection newOccurrences) {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        occurrences.addAll(newOccurrences);
    }

    public Collection makeBinders(Searcher searcher) throws Impossible {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        List binders = new ArrayList();
        Iterator iterator = occurrences.iterator();
        while (iterator.hasNext()) {
            binders.addAll(((Occurrence) iterator.next()).makeBinders(searcher));
        }
        return binders;
    }

    public Collection makeExecutables(VariableMapperCopier variableMapperCopier) {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        Collection executables = new ArrayList();
        Iterator iterator = occurrences.iterator();
        while (iterator.hasNext()) {
            executables.addAll(((Occurrence) iterator.next()).makeExecutables(variableMapperCopier));
        }
        return executables;
    }

    public OccurrenceDescription makeOccurrenceDescription(VariableMapperCopier variableMapperCopier) {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        return null;
    }
}