package de.renew.engine.common;

import de.renew.engine.searcher.AbstractOccurrence;
import de.renew.engine.searcher.OccurrenceDescription;
import de.renew.engine.searcher.Searcher;
import de.renew.engine.searcher.VariableMapperCopier;
import de.renew.engine.simulator.SimulationThreadPool;

import de.renew.expression.Expression;
import de.renew.expression.VariableMapper;

import de.renew.net.TransitionInstance;

import de.renew.unify.Impossible;
import de.renew.unify.Variable;

import java.util.Collection;
import java.util.Collections;


public abstract class EnumeratorOccurrence extends AbstractOccurrence {
    private Expression expression;
    private VariableMapper mapper;
    private boolean checkBound;

    public EnumeratorOccurrence(Expression expression, boolean checkBound,
                                VariableMapper mapper,
                                TransitionInstance tInstance) {
        super(tInstance);
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        this.expression = expression;
        this.checkBound = checkBound;
        this.mapper = mapper;
    }

    public abstract Collection getCandidates(Object pattern);

    public Collection makeBinders(Searcher searcher) throws Impossible {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        Variable var = new Variable(expression.startEvaluation(mapper,
                                                               searcher.recorder,
                                                               searcher.calcChecker),
                                    searcher.recorder);

        return Collections.singleton(new EnumeratorBinder(this, checkBound, var));
    }

    public Collection makeExecutables(VariableMapperCopier copier) {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        return Collections.EMPTY_SET;
    }

    public OccurrenceDescription makeOccurrenceDescription(VariableMapperCopier variableMapperCopier) {
        assert SimulationThreadPool.isSimulationThread() : "is not in a simulation thread";
        return null;
    }
}