package de.renew.util;

import org.apache.log4j.Logger;

import java.io.File;
import java.io.IOException;

import java.net.JarURLConnection;
import java.net.URL;

import java.util.Enumeration;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.filechooser.FileFilter;


/**
 * This class helps finding files. It will take FileFilters filter out the files
 * needed It can also look into jar files and will return the apropriate strings
 *
 * @author Benjamin Schleinzer
 *
 */
public class FileFinder {
    Logger logger = Logger.getLogger(de.renew.util.FileFinder.class);

    // A list of filefilters to check against
    private Vector<FileFilter> fileFilters;

    public FileFinder() {
        fileFilters = new Vector<FileFilter>();
    }

    public void addFileFilter(FileFilter filter) {
        fileFilters.add(filter);
    }

    public void removeFileFilter(FileFilter filter) {
        fileFilters.remove(filter);
    }

    public void search(File directory, Vector<Object> results) {
        for (File entry : directory.listFiles()) {
            if (entry.isDirectory()) {
                search(entry, results);
            } else if (entry.getName().endsWith(".jar")) {
                try {
                    JarFile jar = new JarFile(entry);
                    Enumeration en = jar.entries();
                    while (en.hasMoreElements()) {
                        JarEntry jarEntry = (JarEntry) en.nextElement();
                        File file = new File(jarEntry.getName());
                        for (FileFilter filter : fileFilters) {
                            if (filter.accept(file)) {
                                JarURLConnection jarConnection = (JarURLConnection) new URL("jar:file:"
                                                                                            + entry
                                                                                              .getPath()
                                                                                            + "!/"
                                                                                            + jarEntry
                                                                                              .getName())
                                                                 .openConnection();

                                results.add(jarConnection);
                                break;
                            }
                        }
                    }
                } catch (IOException e) {
                    logger.error("Error during jar file reading");
                }
            } else {
                for (FileFilter filter : fileFilters) {
                    if (filter.accept(entry)) {
                        results.add(entry);
                        break;
                    }
                }
            }
        }
    }

    public void search(final File directory, final FileFindListener listener) {
        new Thread() {
                public void run() {
                    Vector<Object> result = new Vector<Object>();
                    search(directory, result);
                    listener.fileSearchDone(result);
                }
            }.start();
    }
}